/* luafaudes_common.cpp - common functionality of all classes and functions */

/* luafaudes - FAUDES Bindings for Lua

   Copyright (C) 2006 Bernd Opitz
   Exclusive copyright is granted to the Lehrstuhl fuer Regelungstechnik
   at the University of Erlangen-Nuernberg, Germany

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */


#include "luafaudes_common.hpp"

/////////////////////////
// global variables:
/////////////////////////


bool comp_statenames = true;
char* comp_dotpath = "dot";

/////////////////////////////////////
// Runtime configuration functions:
/////////////////////////////////////

int Comp_Statenames_On(lua_State* L) {
	comp_statenames = true;
	return 0;
}

int Comp_Statenames_Off(lua_State* L) {
	comp_statenames = false;
	return 0;
}

int Comp_DotExecPath(lua_State* L) {
        const char* str = luaL_checkstring(L, 1);
 	comp_dotpath = (char*) malloc(strlen(str+1)); // fix mem
        strcpy(comp_dotpath,str); 
	return 0;
}

int Libfaudes_Version(lua_State* L) {
	lua_pushstring(L, FDVersionString().c_str());
	return 1;
}

void RegisterCommonFaudesFunctions(lua_State* L) {
	lua_register(L, "StatenamesOn", Comp_Statenames_On);
	lua_register(L, "StatenamesOff", Comp_Statenames_Off);
	lua_register(L, "DotExecPath", Comp_DotExecPath);
	lua_register(L, "Version", Libfaudes_Version);
}

/////////////////////////////////////
// Helper functions:
/////////////////////////////////////

bool lua_isudata(lua_State* L, int index, const char* tname) {
  void *p = lua_touserdata(L, index);
	if (p == NULL) {
		return false;
	}
	/* get correct metatable */
  lua_getfield(L, LUA_REGISTRYINDEX, tname);  
  if (!lua_getmetatable(L, index)) {
    lua_pop(L, 1);  /* remove both metatables */
    return false;
  }
  else if (!lua_rawequal(L, -1, -2)) {
    lua_pop(L, 2);  /* remove both metatables */
		return false;
	}
  else {
    lua_pop(L, 2);  /* remove both metatables */
    return true;
  }
}

void lua_checkargs(lua_State* L, int min, int max) {
  if(lua_isnoneornil(L,min)) {
    luaL_argerror(L, -1, "not enough arguments");
  } 
  if(!lua_isnoneornil(L,max+1)) {
    luaL_argerror(L, 0, "to many arguments");
  } 
}

