/* luafaudes_gen_class.cpp - Lua bindings for Generator class */

/* luafaudes - FAUDES Bindings for Lua

Copyright (C) 2006 Bernd Opitz
Exclusive copyright is granted to the Lehrstuhl fuer Regelungstechnik
at the University of Erlangen-Nuernberg, Germany

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */



#include "luafaudes_gen_class.hpp"


/////////////////////////////////////////
// Generator new, check and conversion:
/////////////////////////////////////////

bool lua_isgenerator(lua_State* L, int index) {
  return lua_isudata(L, index, LUA_GENERATOR);
}

fGenerator* lua_togenerator(lua_State* L, int index)
{
  // get a userdatum from the lua stack and cast as fGenerator**
  fGenerator** g = (fGenerator**)lua_touserdata(L, index);
  if (g == NULL) luaL_typerror(L, index, LUA_GENERATOR);
  if (*g == NULL) luaL_typerror(L, index, LUA_GENERATOR);
  return *g;
}

fGenerator* luaL_checkgenerator(lua_State* L, int index)
{
  // check type
  if(!lua_isudata(L, index, LUA_GENERATOR)) {
    luaL_typerror(L, index, LUA_GENERATOR);
    return NULL;
  }
  // get reference
  return lua_togenerator(L,index);
}

fGenerator* lua_pushgenerator(lua_State* L)
{
  // allocate memory and push address on the lua stack
  fGenerator** g = (fGenerator**)lua_newuserdata(L, sizeof(fGenerator*));
  // actually construct
  *g = new fGenerator;
  // read from file
  if (lua_isstring(L, 1)) {
    const char* str = luaL_checkstring(L, 1);
    try{
      (*g)->Read(str);
    }
    catch(faudes::Exception& ex){
      std::string errorString = std::string("Error reading file \"");
      errorString = errorString + std::string(str) + std::string("\"");
      DisplayText(errorString);
      (*g)->Clear();
    }
  }
  // from Generator
  else if (lua_isgenerator(L, 1)) {
    fGenerator* g_other = luaL_checkgenerator(L, 1);
    g_other->Copy(**g);
  }
  // default constructor
  else {
  }
  // set computation of statenames to current global value
  if(!comp_statenames) (*g)->ClearStateNames();
  (*g)->StateNamesEnabled(comp_statenames);
  // push the metatable associated with LUA_GENERATOR on the lua stack
  luaL_getmetatable(L, LUA_GENERATOR);
  // pop the table from the stack and set it as the new metatable for the
  // value associated with the fGenerator
  lua_setmetatable(L, -2);
  return *g;
}

int Generator_New(lua_State* L)
{
  lua_pushgenerator(L);
  return 1;
}

int Generator_Help(lua_State* L) {
  std::stringstream sstr;
  sstr
    << std::endl
    << "Generator Methods:" << std::endl
    << "  Generator New()" << std::endl
    << "         nil Copy(Generator)" << std::endl
    << "         nil Read(filename)" << std::endl
    << "         nil Write()" << std::endl
    << "         nil Write(filename)" << std::endl
    << "         nil DWrite()" << std::endl
    << "         nil DotWrite(filename)" << std::endl
    << "         nil DDotWrite(filename)" << std::endl
    << "         nil GraphWrite(filename [,filetype])" << std::endl
    << "         nil StatWrite()" << std::endl
    << "      string AlphabetToString()" << std::endl
    << "      string StatesToString()" << std::endl
    << "      string TransRelToString()" << std::endl
    << "      string InitStatesToString()" << std::endl
    << "      string MarkedStatesToString()" << std::endl
    << "      string Name()" << std::endl
    << "         nil Name(string)" << std::endl
    << "         nil Clear()" << std::endl
    << "         nil ClearStates()" << std::endl
    << "         nil ClearTransRel()" << std::endl
    << "         nil ClearInitStates()" << std::endl
    << "         nil ClearMarkedStates()" << std::endl
    << "      number AlphabetSize()" << std::endl
    << "      number StatesSize()" << std::endl
    << "      number TransRelSize()" << std::endl
    << "      number InitStatesSize()" << std::endl
    << "      number MarkedStatesSize()" << std::endl
    << "     boolean Empty()" << std::endl
    << "     boolean AlphabetEmpty()" << std::endl
    << "     boolean TransRelEmpty()" << std::endl
    << "     boolean InitStatesEmpty()" << std::endl
    << "     boolean MarkedStatesEmpty()" << std::endl
    << "     boolean StateNamesEnabled()" << std::endl
    << "         nil StateNamesEnabled(boolean)" << std::endl
    << "      number InsEvent(string)" << std::endl
    << "         nil InsEvent(number)" << std::endl
    << "         nil InsEvents(EventSet)" << std::endl
    << "         nil DelEvent(string)" << std::endl
    << "         nil DelEvent(number)" << std::endl
    << "         nil DelEvents(EventSet)" << std::endl
    << "         nil InjectAlphabet(EventSet)" << std::endl
    << "      number InsState(string)" << std::endl
    << "      number InsState()" << std::endl
    << "      number InsInitState(string)" << std::endl
    << "      number InsInitState()" << std::endl
    << "      number InsMarkedState(string)" << std::endl
    << "      number InsMarkedState()" << std::endl
    << "         nil SetInitState(number or string)" << std::endl
    << "         nil ClrInitState(number or string)" << std::endl
    << "         nil SetMarkedState(number or string)" << std::endl
    << "         nil ClrMarkedState(number or string)" << std::endl
    << "         nil DelState(number or string)" << std::endl
    << "         nil SetTransition(number or string, number or string, number or string)" << std::endl
    << "         nil ClrTransition(number or string, number or string, number or string)" << std::endl
    << "      string EventName(number)" << std::endl
    << "      number EventIndex(string)" << std::endl
    << "     boolean ExistsEvent(number or string)" << std::endl
    << "      string StateName(number)" << std::endl
    << "      number StateIndex(string)" << std::endl
    << "     boolean ExistsState(number or string)" << std::endl
    << "     boolean ExistsInitState(number or string)" << std::endl
    << "     boolean ExistsMarkedState(number or string)" << std::endl
    << "    EventSet Alphabet()" << std::endl
    << "         nil MinimizeAlphabet()" << std::endl
    << "    EventSet UsedEvents()" << std::endl
    << "    EventSet UnusedEvents()" << std::endl
    << "    EventSet ActiveEventSet(number or string)" << std::endl
    << "         nil SetDefaultStateNames()" << std::endl
    << "         nil ClearStateNames()" << std::endl
    << "         nil Version(Generator, Generator, number or string)" << std::endl
    << "      string EStr(number)" << std::endl
    << "      string SStr(number)" << std::endl
    << "      string TStr(number or string, number or string, number or string)" << std::endl
    << std::endl 
    << "Controllability Methods:" << std::endl
    << "      number InsControllableEvent(string)" << std::endl
    << "         nil InsControllableEvent(number)" << std::endl
    << "      number InsUncontrollableEvent(string)" << std::endl
    << "         nil InsUncontrollableEvent(number)" << std::endl
    << "         nil SetControllable(number or string)" << std::endl
    << "         nil ClrControllable(number or string)" << std::endl
    << "     boolean Controllable(number or string)" << std::endl
    << "    EventSet ControllableEvents()" << std::endl
    << "    EventSet UncontrollableEvents()" << std::endl;
  DisplayText("Generator Userdata Methods", sstr.str());
  return 0;
}



////////////////////////////////
// register Generator methods:
////////////////////////////////


// garbage collection method
int Generator__gc(lua_State* L)
{
  fGenerator* g = lua_togenerator(L, 1);	
  //std::cerr << "garbege collect: " << g->Name() << std::endl;
  delete(g);
  return 0;
} 

// string conversion method (for print())
int Generator__tostring(lua_State* L)
{
  fGenerator* g = lua_togenerator(L,1);
  lua_pushstring(L, g->ToString().c_str());
  return 1;
}

/** 
 * Registration data for LUA_GENERATOR garbage collection and string
 * conversion method
 */
const luaL_reg GeneratorMeta[] = {
  {"__gc",       Generator__gc},
  {"__tostring", Generator__tostring},
  {0, 0}
};

/**
 * Array of functions to be registered by luaL_register
 */
const luaL_reg GeneratorMethods[] = {
  {"New",                   Generator_New},
  {"Help",                  Generator_Help},
  {"Copy",                  Generator_Copy},
  {"Write",                 Generator_Write},
  {"StatWrite",             Generator_StatWrite},
  {"DWrite",                Generator_DWrite},
  {"DotWrite",              Generator_DotWrite},
  {"DDotWrite",             Generator_DDotWrite},
  {"GraphWrite",            Generator_GraphWrite},
  {"AlphabetToString",      Generator_AlphabetToString},
  {"StatesToString",        Generator_StatesToString},
  {"TransRelToString",      Generator_TransRelToString},
  {"InitStatesToString",    Generator_InitStatesToString},
  {"MarkedStatesToString",  Generator_MarkedStatesToString},
  {"Read",                  Generator_Read},
  {"Name",                  Generator_Name},
  {"Clear",                 Generator_Clear},
  {"ClearStates",             Generator_ClearStates},
  {"ClearTransRel",           Generator_ClearTransRel},
  {"ClearInitStates",         Generator_ClearInitStates},
  {"ClearMarkedStates",       Generator_ClearMarkedStates},
  {"AlphabetSize",          Generator_AlphabetSize},
  {"StatesSize",            Generator_Size},
  {"TransRelSize",          Generator_TransRelSize},
  {"InitStatesSize",        Generator_InitStatesSize},
  {"MarkedStatesSize",      Generator_MarkedStatesSize},
  {"Empty",                 Generator_Empty},
  {"AlphabetEmpty",         Generator_AlphabetEmpty},
  {"TransRelEmpty",         Generator_TransRelEmpty},
  {"InitStatesEmpty",       Generator_InitStatesEmpty},
  {"MarkedStatesEmpty",     Generator_MarkedStatesEmpty},
  {"StateNamesEnabled",     Generator_StateNamesEnabled},
  {"InsEvent",              Generator_InsEvent},
  {"InsEvents",             Generator_InsEvents},
  {"DelEvent",              Generator_DelEvent},
  {"DelEvents",             Generator_DelEvents},
  {"InjectAlphabet",        Generator_InjectAlphabet},
  {"InsState",              Generator_InsState},
  {"InsInitState",          Generator_InsInitState},
  {"InsMarkedState",        Generator_InsMarkedState},
  {"SetInitState",          Generator_SetInitState},
  {"ClrInitState",          Generator_ClrInitState},
  {"SetMarkedState",        Generator_SetMarkedState},
  {"ClrMarkedState",        Generator_ClrMarkedState},
  {"DelState",              Generator_DelState},
  {"SetTransition",         Generator_SetTransition},
  {"ClrTransition",         Generator_ClrTransition},
  {"EventName",             Generator_EventName},
  {"EventIndex",            Generator_EventIndex},
  {"ExistsEvent",           Generator_ExistsEvent},
  {"StateName",             Generator_StateName},
  {"StateIndex",            Generator_StateIndex},
  {"ExistsState",           Generator_ExistsState},
  {"ExistsInitState",       Generator_ExistsInitState},
  {"ExistsMarkedState",     Generator_ExistsMarkedState},
  {"Alphabet",              Generator_Alphabet},
  {"MinimizeAlphabet",      Generator_MinimizeAlphabet},
  {"UsedEvents",            Generator_UsedEvents},
  {"UnusedEvents",          Generator_UnusedEvents},
  {"ActiveEventSet",        Generator_ActiveEventSet},
  {"SetDefaultStateNames",  Generator_SetDefaultStateNames},
  {"ClearStateNames",         Generator_ClearStateNames},
  {"Version",               Generator_Version},
  {"EStr",                  Generator_EStr},
  {"SStr",                  Generator_SStr},
  {"TStr",                  Generator_TStr},
  {"InsControllableEvent",  Generator_InsControllableEvent},
  {"InsUncontrollableEvent",Generator_InsUncontrollableEvent},
  {"SetControllable",       Generator_SetControllable},
  {"ClrControllable",       Generator_ClrControllable},
  {"Controllable",          Generator_Controllable},
  {"ControllableEvents",    Generator_ControllableEvents},
  {"UncontrollableEvents",  Generator_UncontrollableEvents},
  {0, 0}
};

///////////////////////////////////////////////
// register Generator data type for oo-Access
///////////////////////////////////////////////

int RegisterGeneratorClass(lua_State* L)
{
  // create methods table add it to the globals
  //luaL_openlib(L, LUA_GENERATOR, GeneratorMethods, 0);
  luaL_register(L, LUA_GENERATOR, GeneratorMethods);
  // create metatable for Generator and add it to the Lua registry 
  luaL_newmetatable(L, LUA_GENERATOR);   
  // fill metatable
  //luaL_openlib(L, 0, GeneratorMeta, 0);
  luaL_register(L, 0, GeneratorMeta);
  lua_pushliteral(L, "__index");
  // dup methods table
  lua_pushvalue(L, -3);
  // metatable.__index = methods
  lua_rawset(L, -3);
  lua_pushliteral(L, "__metatable");
  // dup methods table
  lua_pushvalue(L, -3);
  // hide metatable: metatable.__metatable = methods
  lua_rawset(L, -3);                  
  // drop metatable
  lua_pop(L, 1);        
  // returns methods on the stack

  // additionally register "HelpGenerator()" function for convenience access
  lua_register(L, "HelpGenerator", Generator_Help);
	
  return 2; 
}



///////////////////////
// Generator methods:
///////////////////////



// Copy(g2)
int Generator_Copy(lua_State* L) {
  fGenerator* g1 = luaL_checkgenerator(L, 1);
  fGenerator* g2 = luaL_checkgenerator(L, 2);
  g1->Copy(*g2);
  return 0;
}

// Write() / Write("filename")
int Generator_Write(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isstring(L, 2)) {
    const char* filename = luaL_checkstring(L, 2);
    g->Write(filename);
  }
  else {
    g->Write();
  }
  return 0;
}

// StatWrite() 
int Generator_StatWrite(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->WriteStatistics();
  return 0;
}


// DWrite()
int Generator_DWrite(lua_State* L) {
  lua_getmetatable (L, 1);
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->DWrite();
  return 0;
}

// DotWrite("filename") 
int Generator_DotWrite(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* filename = luaL_checkstring(L, 2);
  g->DotWrite(filename);
  return 0;
}

// DDotWrite("filename") 
int Generator_DDotWrite(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* filename = luaL_checkstring(L, 2);
  g->DDotWrite(filename);
  return 0;
}

// GraphWrite("filename") / GraphWrite("filename", "filetype")
int Generator_GraphWrite(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* filename="";
  const char* filetype="";
  filename = luaL_checkstring(L, 2);
  if (lua_isstring(L, 3)) {
    filetype = luaL_checkstring(L, 2);
  }
  try {
    g->GraphWrite(filename,filetype,comp_dotpath);
  } catch(faudes::Exception& exception) {
    printf("luafaudes::GraphWrite: cannot execute graphviz' dot.\n ");
  }
  return 0;
}


// AlphabetToString()
int Generator_AlphabetToString(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* str = g->AlphabetToString().c_str();
  lua_pushstring(L, str);
  return 1;
}

// StatesToString()
int Generator_StatesToString(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* str = g->StatesToString().c_str();
  lua_pushstring(L, str);
  return 1;
}

// TransRelToString()
int Generator_TransRelToString(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* str = g->TransRelToString().c_str();
  lua_pushstring(L, str);
  return 1;
}

// InitStatesToString()
int Generator_InitStatesToString(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* str = g->InitStatesToString().c_str();
  lua_pushstring(L, str);
  return 1;
}

// MarkedStatesToString()
int Generator_MarkedStatesToString(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* str = g->MarkedStatesToString().c_str();
  lua_pushstring(L, str);
  return 1;
}

// Read("filename") / Read
int Generator_Read(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* filename = luaL_checkstring(L, 2);
  try{
    g->Read(filename);
  }
  catch(faudes::Exception& ex){
    std::string errorString = std::string("Error reading file \"");
    errorString = errorString + std::string(filename) + std::string("\"");
    DisplayText(errorString);
    g->Clear();
  }
  return 0;
}

// Name() / Name("newname")
int Generator_Name(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isstring(L, 2)) {
    const char* name = luaL_checkstring(L, 2);
    g->Name(name);
    return 0;
  }
  else {
    const char* name = g->Name().c_str();
    lua_pushstring(L, name);
    return 1;
  }
}

// Clear()
int Generator_Clear(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->Clear();
  return 0;
}

// ClearStates()
int Generator_ClearStates(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->ClearStates();
  return 0;
}

// ClearTransRel()
int Generator_ClearTransRel(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->ClearTransRel();
  return 0;
}

// ClearInitStates()
int Generator_ClearInitStates(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->ClearInitStates();
  return 0;
}

// ClearMarkedStates()
int Generator_ClearMarkedStates(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->ClearMarkedStates();
  return 0;
}

// AlphabetSize()
int Generator_AlphabetSize(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx size = g->AlphabetSize();
  lua_pushinteger(L, size);
  return 1;
}

// Size()
int Generator_Size(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx size = g->Size();
  lua_pushinteger(L, size);
  return 1;
}

// TransRelSize()
int Generator_TransRelSize(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx size = g->TransRelSize();
  lua_pushinteger(L, size);
  return 1;
}

// InitStatesSize()
int Generator_InitStatesSize(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx size = g->InitStatesSize();
  lua_pushinteger(L, size);
  return 1;
}

// MarkedStatesSize()
int Generator_MarkedStatesSize(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx size = g->MarkedStatesSize();
  lua_pushinteger(L, size);
  return 1;
}

// Empty()
int Generator_Empty(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (g->Empty()) {
    lua_pushboolean(L, 1);
  }
  else {
    lua_pushboolean(L, 0);
  }
  return 1;
}

// AlphabetEmpty()
int Generator_AlphabetEmpty(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (g->AlphabetEmpty()) {
    lua_pushboolean(L, 1);
  }
  else {
    lua_pushboolean(L, 0);
  }
  return 1;
}

// TransRelEmpty()
int Generator_TransRelEmpty(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (g->TransRelEmpty()) {
    lua_pushboolean(L, 1);
  }
  else {
    lua_pushboolean(L, 0);
  }
  return 1;
}

// InitStatesEmpty()
int Generator_InitStatesEmpty(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (g->InitStatesEmpty()) {
    lua_pushboolean(L, 1);
  }
  else {
    lua_pushboolean(L, 0);
  }
  return 1;
}

// MarkedStatesEmpty()
int Generator_MarkedStatesEmpty(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (g->MarkedStatesEmpty()) {
    lua_pushboolean(L, 1);
  }
  else {
    lua_pushboolean(L, 0);
  }
  return 1;
}

// StateNamesEnabled() / StateNamesEnabled(bool)
int Generator_StateNamesEnabled(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isboolean(L, 2)) {
    if (luaL_checkinteger(L, 2)) {
      g->StateNamesEnabled(true);
    }
    else {
      g->StateNamesEnabled(false);
    }
    return 0;
  }
  else {
    if (g->StateNamesEnabled()) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
}

// InsEvent("name") / InsEvent(int)
int Generator_InsEvent(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx event = luaL_checkinteger(L, 2);
    g->InsEvent(event);
    return 0;
  }
  else if (lua_isstring(L, 2)) {
    const char* eventname = luaL_checkstring(L, 2);
    Idx event = g->InsEvent(eventname);
    lua_pushinteger(L, event);
    return 1;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// InsEvents(EventSet)
int Generator_InsEvents(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  fEventSet* e = luaL_checkeventset(L, 2);
  g->InsEvents(*e);
  return 0;
}

// DelEvent("name") / DelEvent(int)
int Generator_DelEvent(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx event = luaL_checkinteger(L, 2);
    g->DelEvent(event);
  }
  else if (lua_isstring(L, 2)) {
    const char* eventname = luaL_checkstring(L, 2);
    g->DelEvent(eventname);
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
  }
  return 0;
}

// DelEvents(EventSet)
int Generator_DelEvents(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  fEventSet* e = luaL_checkeventset(L, 2);
  g->DelEvents(*e);
  return 0;
}

// InjectAlphabet(EventSet)
int Generator_InjectAlphabet(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  fEventSet* e = luaL_checkeventset(L, 2);
  g->InjectAlphabet(*e);
  return 0;
}

// InsState("name") / InsState()
int Generator_InsState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isstring(L, 2)) {
    const char* statename = luaL_checkstring(L, 2);
    Idx stateindex = g->InsState(statename);
    lua_pushinteger(L, stateindex);
  }
  else {
    Idx stateindex = g->InsState();
    lua_pushinteger(L, stateindex);
  }
  return 1;
}

// InsInitState("name") / InsInitState()
int Generator_InsInitState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isstring(L, 2)) {
    const char* statename = luaL_checkstring(L, 2);
    Idx stateindex = g->InsInitState(statename);
    lua_pushinteger(L, stateindex);
  }
  else {
    Idx stateindex = g->InsInitState();
    lua_pushinteger(L, stateindex);
  }
  return 1;
}

// InsMarkedState("name") / InsMarkedState()
int Generator_InsMarkedState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isstring(L, 2)) {
    const char* statename = luaL_checkstring(L, 2);
    Idx stateindex = g->InsMarkedState(statename);
    lua_pushinteger(L, stateindex);
  }
  else {
    Idx stateindex = g->InsMarkedState();
    lua_pushinteger(L, stateindex);
  }
  return 1;
}

// SetInitState("name") / SetInitState(int)
int Generator_SetInitState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx stateindex = luaL_checkinteger(L, 2);
    g->SetInitState(stateindex);
    return 0;
  }
  else if (lua_isstring(L, 2)) {
    const char* statename = luaL_checkstring(L, 2);
    g->SetInitState(statename);
    return 0;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// ClrInitState("name") / ClrInitState(int)
int Generator_ClrInitState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx stateindex = luaL_checkinteger(L, 2);
    g->ClrInitState(stateindex);
    return 0;
  }
  else if (lua_isstring(L, 2)) {
    const char* statename = luaL_checkstring(L, 2);
    g->ClrInitState(statename);
    return 0;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// SetMarkedState("name") / SetMarkedState(int)
int Generator_SetMarkedState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx stateindex = luaL_checkinteger(L, 2);
    g->SetMarkedState(stateindex);
    return 0;
  }
  else if (lua_isstring(L, 2)) {
    const char* statename = luaL_checkstring(L, 2);
    g->SetMarkedState(statename);
    return 0;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// ClrMarkedState("name") / ClrMarkedState(int)
int Generator_ClrMarkedState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx stateindex = luaL_checkinteger(L, 2);
    g->ClrMarkedState(stateindex);
    return 0;
  }
  else if (lua_isstring(L, 2)) {
    const char* statename = luaL_checkstring(L, 2);
    g->ClrMarkedState(statename);
    return 0;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// DelState("name") / DelState(int)
int Generator_DelState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx stateindex = luaL_checkinteger(L, 2);
    g->DelState(stateindex);
  }
  else if (lua_isstring(L, 2)) {
    const char* statename = luaL_checkstring(L, 2);
    g->DelState(statename);
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
  }
  return 0;
}

// SetTransition("x1"/int, "ev"/int, "x2"/int)
int Generator_SetTransition(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx x1, ev, x2;
  if (lua_isnumber(L, 2)) {
    x1 = luaL_checkinteger(L, 2);
  }
  else if (lua_isstring(L, 2)) {
    const char* x1_name = luaL_checkstring(L, 2);
    x1 = g->StateIndex(x1_name);
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }

  if (lua_isnumber(L, 3)) {
    ev = luaL_checkinteger(L, 3);
  }
  else if (lua_isstring(L, 3)) {
    const char* ev_name = luaL_checkstring(L, 3);
    ev = g->EventIndex(ev_name);
  }
  else {
    luaL_typerror(L, 3, "Idx/string");
    return 0;
  }

  if (lua_isnumber(L, 4)) {
    x2 = luaL_checkinteger(L, 4);
  }
  else if (lua_isstring(L, 4)) {
    const char* x2_name = luaL_checkstring(L, 4);
    x2 = g->StateIndex(x2_name);
  }
  else {
    luaL_typerror(L, 4, "Idx/string");
    return 0;
  }
  g->SetTransition(x1, ev, x2);
  return 0;
}

// ClrTransition("x1"/int, "ev"/int, "x2"/int)
int Generator_ClrTransition(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx x1, ev, x2;
  if (lua_isnumber(L, 2)) {
    x1 = luaL_checkinteger(L, 2);
  }
  else if (lua_isstring(L, 2)) {
    const char* x1_name = luaL_checkstring(L, 2);
    x1 = g->StateIndex(x1_name);
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }

  if (lua_isnumber(L, 3)) {
    ev = luaL_checkinteger(L, 3);
  }
  else if (lua_isstring(L, 3)) {
    const char* ev_name = luaL_checkstring(L, 3);
    ev = g->EventIndex(ev_name);
  }
  else {
    luaL_typerror(L, 3, "Idx/string");
    return 0;
  }

  if (lua_isnumber(L, 4)) {
    x2 = luaL_checkinteger(L, 4);
  }
  else if (lua_isstring(L, 4)) {
    const char* x2_name = luaL_checkstring(L, 4);
    x2 = g->StateIndex(x2_name);
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
  g->ClrTransition(x1, ev, x2);
  return 0;
}

// EventName(int)
int Generator_EventName(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx eventindex = luaL_checkinteger(L, 2);
  const char* str = g->EventName(eventindex).c_str();
  lua_pushstring(L, str);
  return 1;
}

// EventIndex("name")
int Generator_EventIndex(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* str = luaL_checkstring(L, 2);
  Idx eventindex = g->EventIndex(str);
  lua_pushinteger(L, eventindex);
  return 1;
}

// ExistsEvent("name") / ExistsEvent(int)
int Generator_ExistsEvent(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx eventindex = luaL_checkinteger(L, 2);
    if (g->ExistsEvent(eventindex)) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
  else if (lua_isstring(L, 2)) {
    const char* eventname = luaL_checkstring(L, 2);
    if (g->ExistsEvent(eventname)) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// StateName(int)
int Generator_StateName(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx index = luaL_checkinteger(L, 2);
  const char* str = g->StateName(index).c_str();
  lua_pushstring(L, str);
  return 1;
}

// StateIndex("name")
int Generator_StateIndex(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  const char* str = luaL_checkstring(L, 2);
  Idx stateindex = g->StateIndex(str);
  lua_pushinteger(L, stateindex);
  return 1;
}

// ExistsState("name") / ExistsState(int)
int Generator_ExistsState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx index = luaL_checkinteger(L, 2);
    if (g->ExistsState(index)) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
  else if (lua_isstring(L, 2)) {
    const char* name = luaL_checkstring(L, 2);
    Idx index = g->StateIndex(name);
    if (index > 0) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// ExistsInitState("name") / ExistsInitState(int)
int Generator_ExistsInitState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx index = luaL_checkinteger(L, 2);
    if (g->ExistsInitState(index)) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
  else if (lua_isstring(L, 2)) {
    const char* name = luaL_checkstring(L, 2);
    Idx index = g->StateIndex(name);
    if (g->ExistsInitState(index)) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// ExistsMarkedState("name") / ExistsMarkedState(int)
int Generator_ExistsMarkedState(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx index = luaL_checkinteger(L, 2);
    if (g->ExistsMarkedState(index)) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
  else if (lua_isstring(L, 2)) {
    const char* name = luaL_checkstring(L, 2);
    Idx index = g->StateIndex(name);
    if (g->ExistsMarkedState(index)) {
      lua_pushboolean(L, 1);
    }
    else {
      lua_pushboolean(L, 0);
    }
    return 1;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// Alphabet()
int Generator_Alphabet(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  fEventSet* e = lua_pusheventset(L);
  *e = g->Alphabet();
  return 1;
}

// MinimizeAlphabet()
int Generator_MinimizeAlphabet(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->MinimizeAlphabet();
  return 0;
}

// UsedEvents() 
int Generator_UsedEvents(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  fEventSet* e = lua_pusheventset(L);
  *e = g->UsedEvents();
  return 1;
}

// UnusedEvents() 
int Generator_UnusedEvents(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  fEventSet* e = lua_pusheventset(L);
  *e = g->UnusedEvents();
  return 1;
}

// ActiveEventSet() 
int Generator_ActiveEventSet(lua_State* L) {
  return Generator_UsedEvents(L);
}

// SetDefaultStateNames() 
int Generator_SetDefaultStateNames(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->SetDefaultStateNames();
  return 0;
}

// ClearStateNames() 
int Generator_ClearStateNames(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  g->ClearStateNames();
  return 0;
}

// g = Version() 
int Generator_Version(lua_State* L) {
  fGenerator* g1 = luaL_checkgenerator(L, 1);
  fGenerator* g2 = luaL_checkgenerator(L, 2);
  if (lua_isnumber(L, 3)) {
    Idx index = luaL_checkinteger(L, 3);
    g1->Version(index, *g2);
  }
  else if (lua_isstring(L, 3)) {
    const char* str = luaL_checkstring(L, 3);
    g1->Version(str, *g2);
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
  }
  return 0;
}

// EStr(Idx)
int Generator_EStr(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx index = luaL_checkinteger(L, 2);
  const char* str = g->EStr(index).c_str();
  lua_pushstring(L, str);
  return 1;
}

// SStr(Idx)
int Generator_SStr(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx index = luaL_checkinteger(L, 2);
  const char* str = g->SStr(index).c_str();
  lua_pushstring(L, str);
  return 1;
}


// TStr(Idx)
int Generator_TStr(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx x1, ev, x2;
  if (lua_isnumber(L, 2)) {
    x1 = luaL_checkinteger(L, 2);
  }
  else if (lua_isstring(L, 2)) {
    const char* x1_name = luaL_checkstring(L, 2);
    x1 = g->StateIndex(x1_name);
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
  if (lua_isnumber(L, 3)) {
    ev = luaL_checkinteger(L, 3);
  }
  else if (lua_isstring(L, 3)) {
    const char* ev_name = luaL_checkstring(L, 3);
    ev = g->EventIndex(ev_name);
  }
  else {
    luaL_typerror(L, 3, "Idx/string");
    return 0;
  }
  if (lua_isnumber(L, 4)) {
    x2 = luaL_checkinteger(L, 4);
  }
  else if (lua_isstring(L, 4)) {
    const char* x2_name = luaL_checkstring(L, 4);
    x2 = g->StateIndex(x2_name);
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
  g->TStr(Transition(x1, ev, x2));
  return 0;
}


// InsControllableEvent("name") / InsControllableEvent(int)
int Generator_InsControllableEvent(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx event = luaL_checkinteger(L, 2);
    g->InsControllableEvent(event);
    return 0;
  }
  else if (lua_isstring(L, 2)) {
    const char* eventname = luaL_checkstring(L, 2);
    Idx event = g->InsControllableEvent(eventname);
    lua_pushinteger(L, event);
    return 1;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// InsUncontrollableEvent("name") / InsUncontrollableEvent(int)
int Generator_InsUncontrollableEvent(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx event = luaL_checkinteger(L, 2);
    g->InsUncontrollableEvent(event);
    return 0;
  }
  else if (lua_isstring(L, 2)) {
    const char* eventname = luaL_checkstring(L, 2);
    Idx event = g->InsUncontrollableEvent(eventname);
    lua_pushinteger(L, event);
    return 1;
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
}

// SetControllable("name") / SetControllable(int)
int Generator_SetControllable(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx event = luaL_checkinteger(L, 2);
    g->SetControllable(event);
  }
  else if (lua_isstring(L, 2)) {
    const char* eventname = luaL_checkstring(L, 2);
    g->SetControllable(std::string(eventname) );
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
  }
  return 0;
}

// ClrControllable("name") / ClrControllable(int)
int Generator_ClrControllable(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  if (lua_isnumber(L, 2)) {
    Idx event = luaL_checkinteger(L, 2);
    g->ClrControllable(event);
  }
  else if (lua_isstring(L, 2)) {
    const char* eventname = luaL_checkstring(L, 2);
    g->ClrControllable(std::string(eventname) );
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
  }
  return 0;
}

// Controllable("name") / Controllable(int)
int Generator_Controllable(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  Idx event;
  if (lua_isnumber(L, 2)) {
    event = luaL_checkinteger(L, 2);
  }
  else if (lua_isstring(L, 2)) {
    const char* eventname = luaL_checkstring(L, 2);
    event = g->EventIndex(eventname);
    if (event == 0) {
      luaL_error(L, "event does not exist");
      return 0;
    }
  }
  else {
    luaL_typerror(L, 2, "Idx/string");
    return 0;
  }
  if (g->Controllable(event)) {
    lua_pushboolean(L, 1);
  }
  else {
    lua_pushboolean(L, 0);
  }
  return 1;
}

// ControllableEvents()
int Generator_ControllableEvents(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  fEventSet* e = lua_pusheventset(L);
  *e = g->ControllableEvents();
  return 1;
}

// UncontrollableEvents()
int Generator_UncontrollableEvents(lua_State* L) {
  fGenerator* g = luaL_checkgenerator(L, 1);
  fEventSet* e = lua_pusheventset(L);
  *e = g->UncontrollableEvents();
  return 1;
}



